# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist, constr
from lusid_workflow.models.action_definition import ActionDefinition
from lusid_workflow.models.initial_state import InitialState
from lusid_workflow.models.task_field_definition import TaskFieldDefinition
from lusid_workflow.models.task_state_definition import TaskStateDefinition
from lusid_workflow.models.task_transition_definition import TaskTransitionDefinition
from lusid_workflow.models.transition_trigger_definition import TransitionTriggerDefinition

class UpdateTaskDefinitionRequest(BaseModel):
    """
    Contains required info to update a Task Definition  # noqa: E501
    """
    display_name: constr(strict=True, min_length=1) = Field(..., alias="displayName", description="Human readable name")
    description: Optional[StrictStr] = Field(None, description="Human readable description")
    states: conlist(TaskStateDefinition, min_items=1) = Field(..., description="The states this Task Definition operates over")
    field_schema: Optional[conlist(TaskFieldDefinition)] = Field(None, alias="fieldSchema", description="Defines the fields associated with this Task")
    initial_state: InitialState = Field(..., alias="initialState")
    triggers: Optional[conlist(TransitionTriggerDefinition)] = Field(None, description="Triggers")
    transitions: Optional[conlist(TaskTransitionDefinition)] = Field(None, description="Transitions")
    actions: Optional[conlist(ActionDefinition)] = Field(None, description="Actions")
    __properties = ["displayName", "description", "states", "fieldSchema", "initialState", "triggers", "transitions", "actions"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpdateTaskDefinitionRequest:
        """Create an instance of UpdateTaskDefinitionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in states (list)
        _items = []
        if self.states:
            for _item in self.states:
                if _item:
                    _items.append(_item.to_dict())
            _dict['states'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in field_schema (list)
        _items = []
        if self.field_schema:
            for _item in self.field_schema:
                if _item:
                    _items.append(_item.to_dict())
            _dict['fieldSchema'] = _items
        # override the default output from pydantic by calling `to_dict()` of initial_state
        if self.initial_state:
            _dict['initialState'] = self.initial_state.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in triggers (list)
        _items = []
        if self.triggers:
            for _item in self.triggers:
                if _item:
                    _items.append(_item.to_dict())
            _dict['triggers'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in transitions (list)
        _items = []
        if self.transitions:
            for _item in self.transitions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['transitions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in actions (list)
        _items = []
        if self.actions:
            for _item in self.actions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['actions'] = _items
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if field_schema (nullable) is None
        # and __fields_set__ contains the field
        if self.field_schema is None and "field_schema" in self.__fields_set__:
            _dict['fieldSchema'] = None

        # set to None if triggers (nullable) is None
        # and __fields_set__ contains the field
        if self.triggers is None and "triggers" in self.__fields_set__:
            _dict['triggers'] = None

        # set to None if transitions (nullable) is None
        # and __fields_set__ contains the field
        if self.transitions is None and "transitions" in self.__fields_set__:
            _dict['transitions'] = None

        # set to None if actions (nullable) is None
        # and __fields_set__ contains the field
        if self.actions is None and "actions" in self.__fields_set__:
            _dict['actions'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpdateTaskDefinitionRequest:
        """Create an instance of UpdateTaskDefinitionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpdateTaskDefinitionRequest.parse_obj(obj)

        _obj = UpdateTaskDefinitionRequest.parse_obj({
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "states": [TaskStateDefinition.from_dict(_item) for _item in obj.get("states")] if obj.get("states") is not None else None,
            "field_schema": [TaskFieldDefinition.from_dict(_item) for _item in obj.get("fieldSchema")] if obj.get("fieldSchema") is not None else None,
            "initial_state": InitialState.from_dict(obj.get("initialState")) if obj.get("initialState") is not None else None,
            "triggers": [TransitionTriggerDefinition.from_dict(_item) for _item in obj.get("triggers")] if obj.get("triggers") is not None else None,
            "transitions": [TaskTransitionDefinition.from_dict(_item) for _item in obj.get("transitions")] if obj.get("transitions") is not None else None,
            "actions": [ActionDefinition.from_dict(_item) for _item in obj.get("actions")] if obj.get("actions") is not None else None
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, constr, validator

class TaskTransitionDefinition(BaseModel):
    """
    Defines a State change  # noqa: E501
    """
    from_state: constr(strict=True, max_length=1024, min_length=1) = Field(..., alias="fromState", description="The State this Transition if coming From")
    to_state: constr(strict=True, max_length=1024, min_length=1) = Field(..., alias="toState", description="The State this Transition is going To")
    trigger: constr(strict=True, max_length=1024, min_length=1) = Field(..., description="The Trigger for this Transition")
    guard: Optional[constr(strict=True, max_length=1024, min_length=1)] = Field(None, description="The Guard for this Transition, if any")
    action: Optional[constr(strict=True, max_length=1024, min_length=1)] = Field(None, description="The Action to invoke on successful Transition")
    __properties = ["fromState", "toState", "trigger", "guard", "action"]

    @validator('from_state')
    def from_state_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z0-9\-_]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9\-_]+$/")
        return value

    @validator('to_state')
    def to_state_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z0-9\-_]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9\-_]+$/")
        return value

    @validator('trigger')
    def trigger_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z0-9\-_]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9\-_]+$/")
        return value

    @validator('guard')
    def guard_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[\s\S]*$", value):
            raise ValueError(r"must validate the regular expression /^[\s\S]*$/")
        return value

    @validator('action')
    def action_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[a-zA-Z0-9\-_]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9\-_]+$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TaskTransitionDefinition:
        """Create an instance of TaskTransitionDefinition from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if guard (nullable) is None
        # and __fields_set__ contains the field
        if self.guard is None and "guard" in self.__fields_set__:
            _dict['guard'] = None

        # set to None if action (nullable) is None
        # and __fields_set__ contains the field
        if self.action is None and "action" in self.__fields_set__:
            _dict['action'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TaskTransitionDefinition:
        """Create an instance of TaskTransitionDefinition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TaskTransitionDefinition.parse_obj(obj)

        _obj = TaskTransitionDefinition.parse_obj({
            "from_state": obj.get("fromState"),
            "to_state": obj.get("toState"),
            "trigger": obj.get("trigger"),
            "guard": obj.get("guard"),
            "action": obj.get("action")
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist
from lusid_workflow.models.link import Link

class DeletedEntityResponse(BaseModel):
    """
    DeletedEntityResponse
    """
    href: Optional[StrictStr] = Field(None, description="The Uri related to this Entity")
    effective_from: Optional[datetime] = Field(None, alias="effectiveFrom", description="The EffectiveFrom for this response")
    as_at: datetime = Field(..., alias="asAt", description="The AsAt for this response")
    links: Optional[conlist(Link)] = None
    __properties = ["href", "effectiveFrom", "asAt", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> DeletedEntityResponse:
        """Create an instance of DeletedEntityResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict['href'] = None

        # set to None if effective_from (nullable) is None
        # and __fields_set__ contains the field
        if self.effective_from is None and "effective_from" in self.__fields_set__:
            _dict['effectiveFrom'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> DeletedEntityResponse:
        """Create an instance of DeletedEntityResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return DeletedEntityResponse.parse_obj(obj)

        _obj = DeletedEntityResponse.parse_obj({
            "href": obj.get("href"),
            "effective_from": obj.get("effectiveFrom"),
            "as_at": obj.get("asAt"),
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, StrictStr

class WorkerStatusTriggers(BaseModel):
    """
    Defines triggers that will be invoked per Worker outcome  # noqa: E501
    """
    started: Optional[StrictStr] = Field(None, description="Trigger to invoke when the Worker has Started")
    completed_with_results: Optional[StrictStr] = Field(None, alias="completedWithResults", description="Trigger to invoke when the Worker has Completed (with results)")
    completed_no_results: Optional[StrictStr] = Field(None, alias="completedNoResults", description="Trigger to invoke when the Worker has Completed (no results)")
    failed_to_start: Optional[StrictStr] = Field(None, alias="failedToStart", description="Trigger to invoke when the Worker has Failed to Start")
    failed_to_complete: Optional[StrictStr] = Field(None, alias="failedToComplete", description="Trigger to invoke when the Worker has Failed to Complete")
    __properties = ["started", "completedWithResults", "completedNoResults", "failedToStart", "failedToComplete"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> WorkerStatusTriggers:
        """Create an instance of WorkerStatusTriggers from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if started (nullable) is None
        # and __fields_set__ contains the field
        if self.started is None and "started" in self.__fields_set__:
            _dict['started'] = None

        # set to None if completed_with_results (nullable) is None
        # and __fields_set__ contains the field
        if self.completed_with_results is None and "completed_with_results" in self.__fields_set__:
            _dict['completedWithResults'] = None

        # set to None if completed_no_results (nullable) is None
        # and __fields_set__ contains the field
        if self.completed_no_results is None and "completed_no_results" in self.__fields_set__:
            _dict['completedNoResults'] = None

        # set to None if failed_to_start (nullable) is None
        # and __fields_set__ contains the field
        if self.failed_to_start is None and "failed_to_start" in self.__fields_set__:
            _dict['failedToStart'] = None

        # set to None if failed_to_complete (nullable) is None
        # and __fields_set__ contains the field
        if self.failed_to_complete is None and "failed_to_complete" in self.__fields_set__:
            _dict['failedToComplete'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> WorkerStatusTriggers:
        """Create an instance of WorkerStatusTriggers from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return WorkerStatusTriggers.parse_obj(obj)

        _obj = WorkerStatusTriggers.parse_obj({
            "started": obj.get("started"),
            "completed_with_results": obj.get("completedWithResults"),
            "completed_no_results": obj.get("completedNoResults"),
            "failed_to_start": obj.get("failedToStart"),
            "failed_to_complete": obj.get("failedToComplete")
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict
from pydantic.v1 import BaseModel, Field, constr

class ActionId(BaseModel):
    """
    ActionId
    """
    scope: constr(strict=True, max_length=100, min_length=3) = Field(...)
    activity: constr(strict=True, max_length=25, min_length=3) = Field(...)
    entity: constr(strict=True, max_length=40, min_length=3) = Field(...)
    __properties = ["scope", "activity", "entity"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ActionId:
        """Create an instance of ActionId from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ActionId:
        """Create an instance of ActionId from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ActionId.parse_obj(obj)

        _obj = ActionId.parse_obj({
            "scope": obj.get("scope"),
            "activity": obj.get("activity"),
            "entity": obj.get("entity")
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, constr

class ActionLogOrigin(BaseModel):
    """
    The Action Log Origin contains information about how the Action was created  # noqa: E501
    """
    task_id: Optional[StrictStr] = Field(None, alias="taskId", description="The Id of the Task that created this Action")
    request_id: constr(strict=True, min_length=1) = Field(..., alias="requestId", description="The request Id of the request that caused this Action to be triggered.  This could be the original request that caused a sequence of Actions that resulted in this Action")
    __properties = ["taskId", "requestId"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ActionLogOrigin:
        """Create an instance of ActionLogOrigin from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if task_id (nullable) is None
        # and __fields_set__ contains the field
        if self.task_id is None and "task_id" in self.__fields_set__:
            _dict['taskId'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ActionLogOrigin:
        """Create an instance of ActionLogOrigin from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ActionLogOrigin.parse_obj(obj)

        _obj = ActionLogOrigin.parse_obj({
            "task_id": obj.get("taskId"),
            "request_id": obj.get("requestId")
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, conlist, constr, validator
from lusid_workflow.models.action_log_item import ActionLogItem
from lusid_workflow.models.action_log_origin import ActionLogOrigin

class ActionLog(BaseModel):
    """
    An Action Log contains the processing history of an Action  # noqa: E501
    """
    id: constr(strict=True) = Field(..., description="Unique identifier of the Action")
    origin: ActionLogOrigin = Field(...)
    action_type: constr(strict=True, min_length=1) = Field(..., alias="actionType", description="The type of the Action")
    run_as_user_id: Optional[constr(strict=True, max_length=1024)] = Field(None, alias="runAsUserId", description="The ID of the user that the Action was performed by.  If not specified, the actions were performed by the \"current user\".")
    logged_items: conlist(ActionLogItem) = Field(..., alias="loggedItems", description="The logged items for this Action")
    __properties = ["id", "origin", "actionType", "runAsUserId", "loggedItems"]

    @validator('id')
    def id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not re.match(r"^[a-zA-Z0-9\-]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9\-]+$/")
        return value

    @validator('run_as_user_id')
    def run_as_user_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[a-zA-Z0-9\-_]+$", value):
            raise ValueError(r"must validate the regular expression /^[a-zA-Z0-9\-_]+$/")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ActionLog:
        """Create an instance of ActionLog from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of origin
        if self.origin:
            _dict['origin'] = self.origin.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in logged_items (list)
        _items = []
        if self.logged_items:
            for _item in self.logged_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['loggedItems'] = _items
        # set to None if run_as_user_id (nullable) is None
        # and __fields_set__ contains the field
        if self.run_as_user_id is None and "run_as_user_id" in self.__fields_set__:
            _dict['runAsUserId'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ActionLog:
        """Create an instance of ActionLog from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ActionLog.parse_obj(obj)

        _obj = ActionLog.parse_obj({
            "id": obj.get("id"),
            "origin": ActionLogOrigin.from_dict(obj.get("origin")) if obj.get("origin") is not None else None,
            "action_type": obj.get("actionType"),
            "run_as_user_id": obj.get("runAsUserId"),
            "logged_items": [ActionLogItem.from_dict(_item) for _item in obj.get("loggedItems")] if obj.get("loggedItems") is not None else None
        })
        return _obj



import collections
import datetime
import logging

logger = logging.getLogger(__name__)


class FileAccessToken(collections.UserString):
    """Loads access token from file when requested
    Acts as a string so can be concatenated to auth headers
    """

    def __init__(self, access_token_location: str, expiry_time:int = 120):
        if access_token_location is None or access_token_location == "":
            raise ValueError("access_token_location must be a non-empty string")
        self.__access_token_location = access_token_location
        self.__expiry_time = expiry_time 
        self.__token_data = {
            "expires": datetime.datetime.now(),
            "current_access_token": "",
        }

    @property
    def data(self) -> str:
        """load access token from file

        Returns
        -------
        str
            Access token
        """
        if self.__token_data["expires"] <= datetime.datetime.now():
            try:
                with open(self.__access_token_location, "r") as access_token_file:
                    self.__token_data["current_access_token"] = access_token_file.read()
                self.__token_data["expires"] = (
                    datetime.datetime.now() + datetime.timedelta(seconds=self.__expiry_time)
                )
            except OSError:
                logger.error("Could not open access token file")
                raise
        return self.__token_data["current_access_token"]



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist, constr

class GetWorkerResultResponse(BaseModel):
    """
    The RunWorker response  # noqa: E501
    """
    worker_status: constr(strict=True, min_length=1) = Field(..., alias="workerStatus", description="The final status of the Worker")
    results: conlist(Dict[str, Any]) = Field(..., description="Results")
    status_detail: Optional[StrictStr] = Field(None, alias="statusDetail", description="Detail on the final status")
    __properties = ["workerStatus", "results", "statusDetail"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GetWorkerResultResponse:
        """Create an instance of GetWorkerResultResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if status_detail (nullable) is None
        # and __fields_set__ contains the field
        if self.status_detail is None and "status_detail" in self.__fields_set__:
            _dict['statusDetail'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> GetWorkerResultResponse:
        """Create an instance of GetWorkerResultResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return GetWorkerResultResponse.parse_obj(obj)

        _obj = GetWorkerResultResponse.parse_obj({
            "worker_status": obj.get("workerStatus"),
            "results": obj.get("results"),
            "status_detail": obj.get("statusDetail")
        })
        return _obj



# flake8: noqa

# import apis into api package
from lusid_workflow.api.action_logs_api import ActionLogsApi
from lusid_workflow.api.application_metadata_api import ApplicationMetadataApi
from lusid_workflow.api.event_handlers_api import EventHandlersApi
from lusid_workflow.api.task_definitions_api import TaskDefinitionsApi
from lusid_workflow.api.tasks_api import TasksApi
from lusid_workflow.api.workers_api import WorkersApi


__all__ = [
    "ActionLogsApi",
    "ApplicationMetadataApi",
    "EventHandlersApi",
    "TaskDefinitionsApi",
    "TasksApi",
    "WorkersApi"
]



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist
from lusid_workflow.models.task_instance_field import TaskInstanceField

class UpdateTaskRequest(BaseModel):
    """
    A request to update a Task  # noqa: E501
    """
    correlation_ids: Optional[conlist(StrictStr)] = Field(None, alias="correlationIds", description="A set of guid identifiers that allow correlation across the application tier")
    fields: Optional[conlist(TaskInstanceField)] = Field(None, description="Defines the fields associated with the update")
    stacking_key: Optional[StrictStr] = Field(None, alias="stackingKey", description="The key for the Stack that this Task should be added to")
    __properties = ["correlationIds", "fields", "stackingKey"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UpdateTaskRequest:
        """Create an instance of UpdateTaskRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in fields (list)
        _items = []
        if self.fields:
            for _item in self.fields:
                if _item:
                    _items.append(_item.to_dict())
            _dict['fields'] = _items
        # set to None if correlation_ids (nullable) is None
        # and __fields_set__ contains the field
        if self.correlation_ids is None and "correlation_ids" in self.__fields_set__:
            _dict['correlationIds'] = None

        # set to None if fields (nullable) is None
        # and __fields_set__ contains the field
        if self.fields is None and "fields" in self.__fields_set__:
            _dict['fields'] = None

        # set to None if stacking_key (nullable) is None
        # and __fields_set__ contains the field
        if self.stacking_key is None and "stacking_key" in self.__fields_set__:
            _dict['stackingKey'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UpdateTaskRequest:
        """Create an instance of UpdateTaskRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UpdateTaskRequest.parse_obj(obj)

        _obj = UpdateTaskRequest.parse_obj({
            "correlation_ids": obj.get("correlationIds"),
            "fields": [TaskInstanceField.from_dict(_item) for _item in obj.get("fields")] if obj.get("fields") is not None else None,
            "stacking_key": obj.get("stackingKey")
        })
        return _obj



# coding: utf-8

"""
    FINBOURNE Workflow API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from typing import Any, Dict, Optional
from pydantic.v1 import BaseModel, Field, StrictInt, constr

class ChangeItem(BaseModel):
    """
    Defines a change that occured to a Task  # noqa: E501
    """
    as_at_modified: datetime = Field(..., alias="asAtModified", description="The AsAt time of the change")
    user_id_modified: constr(strict=True, min_length=1) = Field(..., alias="userIdModified", description="The User ID that performed the change")
    request_id_modified: constr(strict=True, min_length=1) = Field(..., alias="requestIdModified", description="The Request ID of the request that caused the change")
    as_at_version_number: StrictInt = Field(..., alias="asAtVersionNumber", description="The AsAt Version number")
    action: constr(strict=True, min_length=1) = Field(..., description="The Action that resulted in the change")
    attribute_name: constr(strict=True, min_length=1) = Field(..., alias="attributeName", description="The name of the attribute that has been change")
    previous_value: Optional[Any] = Field(None, alias="previousValue", description="The value of the attribute prior to the change")
    new_value: Optional[Any] = Field(..., alias="newValue", description="The value of the attribute following the change")
    __properties = ["asAtModified", "userIdModified", "requestIdModified", "asAtVersionNumber", "action", "attributeName", "previousValue", "newValue"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ChangeItem:
        """Create an instance of ChangeItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if previous_value (nullable) is None
        # and __fields_set__ contains the field
        if self.previous_value is None and "previous_value" in self.__fields_set__:
            _dict['previousValue'] = None

        # set to None if new_value (nullable) is None
        # and __fields_set__ contains the field
        if self.new_value is None and "new_value" in self.__fields_set__:
            _dict['newValue'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ChangeItem:
        """Create an instance of ChangeItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ChangeItem.parse_obj(obj)

        _obj = ChangeItem.parse_obj({
            "as_at_modified": obj.get("asAtModified"),
            "user_id_modified": obj.get("userIdModified"),
            "request_id_modified": obj.get("requestIdModified"),
            "as_at_version_number": obj.get("asAtVersionNumber"),
            "action": obj.get("action"),
            "attribute_name": obj.get("attributeName"),
            "previous_value": obj.get("previousValue"),
            "new_value": obj.get("newValue")
        })
        return _obj
